/***************************************************************************

	filer.c

	File browser

***************************************************************************/

#include "ncdz.h"

#define MAX_ENTRY 1024


/***************************************************************************
	Global variables
 ***************************************************************************/

char launchDir[MAX_PATH];
char game_dir[MAX_PATH];
char mp3_dir[MAX_PATH];


/***************************************************************************
	Private structures
 ***************************************************************************/

struct dirent
{
	u32 type;
	int icon;
	int has_title;
	char name[256];
};

static struct dirent files[MAX_ENTRY];
static struct dirent file;
static SceIoDirent dir;


/***************************************************************************
	Private variables
 ***************************************************************************/

static char curr_dir[MAX_PATH];
static int nfiles;
static int neocddir;
static int title_exist;
static u16 title_img[80][144];


////////////////////////////////////////////////////////////////////////
// NCbN\[g
int cmpFile(struct dirent *a, struct dirent *b)
{
	char ca, cb;
	int i, n, ret;

	if(a->type==b->type){
		n=strlen(a->name);
		for(i=0; i<=n; i++){
			ca=a->name[i]; cb=b->name[i];
			if(ca>='a' && ca<='z') ca-=0x20;
			if(cb>='a' && cb<='z') cb-=0x20;

			ret = ca-cb;
			if(ret!=0) return ret;
		}
		return 0;
	}

	if(a->type & FIO_SO_IFDIR)	return -1;
	else					return 1;
}

void sort(struct dirent *a, int left, int right)
{
	struct dirent tmp, pivot;
	int i, p;

	if (left < right) {
		pivot = a[left];
		p = left;
		for (i=left+1; i<=right; i++) {
			if (cmpFile(&a[i],&pivot)<0){
				p=p+1;
				tmp=a[p];
				a[p]=a[i];
				a[i]=tmp;
			}
		}
		a[left] = a[p];
		a[p] = pivot;
		sort(a, left, p-1);
		sort(a, p+1, right);
	}
}


/*--------------------------------------------------------
	Find a file
 -------------------------------------------------------*/

char *find_file(char *pattern, char *path)
{
	int fd, i, found, len1, len2;

	memset(&dir, 0, sizeof(dir));

	fd = sceIoDopen(path);
	found = 0;

	len1 = strlen(pattern);

	while (!found)
	{
		if (sceIoDread(fd, &dir) <= 0) break;

		len2 = strlen(dir.d_name);

		for (i = 0; i < len2; i++)
		{
			if (strnicmp(&dir.d_name[i], pattern, len1) == 0)
			{
				strcpy(file.name, dir.d_name);
				found = 1;
				break;
			}
		}
	}

	sceIoDclose(fd);

	return found ? file.name : NULL;
}


/*--------------------------------------------------------
	Find state file
 -------------------------------------------------------*/

#ifdef SAVE_STATE

void find_state_file(u8 *slot)
{
	int fd, len1, len2;
	char path[MAX_PATH], pattern[16];

	memset(&dir, 0, sizeof(dir));

	sprintf(path, "%sstate", launchDir);
	sprintf(pattern, "%s.sv", game_name);

	len1 = strlen(pattern);
	fd = sceIoDopen(path);

	while (sceIoDread(fd, &dir) > 0)
	{
		if (strnicmp(dir.d_name, pattern, len1) == 0)
		{
			char number = dir.d_name[len1] - '0';

			if (number >= 0 && number <= 9)
				slot[number] = 1;
		}
	}

	sceIoDclose(fd);
}

#endif


/*--------------------------------------------------------
	Check directories
 -------------------------------------------------------*/

static void checkDir(const char *name)
{
	int fd, found;
	char path[MAX_PATH];

	memset(&dir, 0, sizeof(dir));

	fd = sceIoDopen(launchDir);
	found = 0;

	while (!found)
	{
		if (sceIoDread(fd, &dir) <= 0) break;

		if (dir.d_stat.st_attr == FIO_SO_IFDIR)
			if (stricmp(dir.d_name, name) == 0)
				found = 1;
	}

	sceIoDclose(fd);

	if (!found)
	{
		sprintf(path, "%s%s", launchDir, name);
		sceIoMkdir(path, 0777);
	}
}


/*--------------------------------------------------------
	Get directory entry
 -------------------------------------------------------*/

static void getDir(const char *path)
{
	static int fd;
	int b = 0;

	memset(&dir, 0, sizeof(dir));

	nfiles = 0;

	if (strcmp(path, "ms0:/") != 0)
	{
		strcpy(files[nfiles].name, "..");
		files[nfiles].type = FIO_SO_IFDIR;
		files[nfiles].icon = ICON_S_UPPERDIR;
		files[nfiles].has_title = 0;
		nfiles++;
		b = 1;
	}

	fd = sceIoDopen(path);

	while (nfiles < MAX_ENTRY)
	{
		char *ext;
		static SceIoDirent dir = { 0 };

		if (sceIoDread(fd, &dir) <= 0) break;

		if (dir.d_name[0] == '.') continue;

		if (stricmp(dir.d_name, "ipl.txt") == 0)
		{
			neocddir = 1;
			continue;
		}

		if ((ext = strrchr(dir.d_name, '.')) != NULL)
		{
			if (stricmp(ext, ".zip") == 0)
			{
				strcpy(files[nfiles].name, dir.d_name);
				files[nfiles].type = 0;
				files[nfiles].icon = ICON_S_ZIPFILE;
				files[nfiles].has_title = 0;
				nfiles++;
				continue;
			}
		}

		if (dir.d_stat.st_attr == FIO_SO_IFDIR)
		{
			if (stricmp(dir.d_name, "config") == 0) continue;
			if (stricmp(dir.d_name, "snap") == 0) continue;
#ifdef SAVE_STATE
			if (stricmp(dir.d_name, "state") == 0) continue;
#endif

			strcpy(files[nfiles].name, dir.d_name);
			strcat(files[nfiles].name, "/");
			files[nfiles].type = FIO_SO_IFDIR;
			files[nfiles].icon = ICON_S_FOLDER;
			files[nfiles].has_title = 1;
			nfiles++;
		}
	}

	sceIoDclose(fd);

	if (b)
		sort(files + 1, 0, nfiles - 2);
	else
		sort(files, 0, nfiles - 1);
}


/*--------------------------------------------------------
	Draw title_x.sys to title image buffer
 -------------------------------------------------------*/

static void title_draw_spr(int sx, int sy, u8 *spr, u16 *pal, int tileno)
{
	int y;
	u32 tile;
	u32 *gfx = (u32 *)(spr + tileno * 128);
	u16 *paldata = &pal[tileno << 4];

	for (y = sy; y < sy + 16; y++)
	{
		tile = *gfx++;
		title_img[y][sx +  0] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  1] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  2] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  3] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  4] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  5] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  6] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  7] = paldata[tile & 0x0f];
		tile = *gfx++;
		title_img[y][sx +  8] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx +  9] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 10] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 11] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 12] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 13] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 14] = paldata[tile & 0x0f]; tile >>= 4;
		title_img[y][sx + 15] = paldata[tile & 0x0f];
	}
}


/*--------------------------------------------------------
	Load title_x.sys
 -------------------------------------------------------*/

static int load_title(const char *path)
{
	int i, region, tileno, x, y, found = 0;
	u16 title_pal[0x5a0 >> 1];
	u8  title_spr[0x1680];
	char title_path[MAX_PATH], region_chr[3] = {'j','u','e'};
	FILE *fp;

	for (region = neogeo_region & 0x03; region >= 0; region--)
	{
		sprintf(title_path, "%stitle_%c.sys", path, region_chr[region]);

		if ((fp = fopen(title_path, "rb")) != NULL)
		{
			found = 1;
			break;
		}
	}

	if (!found) return 0;

	fread(title_pal, 0x5a0, 1, fp);
	fread(title_spr, 0x1680, 1, fp);
	fclose(fp);

	swab((u8 *)title_pal, (u8 *)title_pal, 0x5a0);

	for (i = 0; i < 0x5a0 >> 1; i++)
	{
		int r = ((title_pal[i] >> 7) & 0x1e) | ((title_pal[i] >> 14) & 0x01);
		int g = ((title_pal[i] >> 3) & 0x1e) | ((title_pal[i] >> 13) & 0x01);
		int b = ((title_pal[i] << 1) & 0x1e) | ((title_pal[i] >> 12) & 0x01);

		title_pal[i] = (b << 10) | (g << 5) | r;
	}

	neogeo_decode_spr(title_spr, 0, 0x1680);

	tileno = 0;

	for (y = 0; y < 80; y += 16)
	{
		for (x = 0; x < 144; x += 16)
		{
			title_draw_spr(x, y, title_spr, title_pal, tileno);
			tileno++;
		}
	}

	return 1;
}


/*--------------------------------------------------------
	Show title_x.sys
 -------------------------------------------------------*/

static void show_title(int sx, int sy)
{
	u16 *dst = video_draw_frame(sx, sy);
	int x, y;

	for (y = 0; y < 80; y++)
	{
		for (x = 0; x < 144; x++)
		{
			dst[x] = title_img[y][x];
		}

		dst += BUF_WIDTH;
	}
}


/*--------------------------------------------------------
	Run filer
 -------------------------------------------------------*/

#define TITLE_COL		MAKECOL15(255,255,120)
#define URL_COL			MAKECOL15(220,220,220)
#define PATH_COL		MAKECOL15(200,200,255)
#define INFO_COL 		MAKECOL15(255,255,128)
#define WARNING_COL 	MAKECOL15(255,128,128)
#define INFOBAR_COL 	MAKECOL15(220,220,220)
#define SELECT_COL		MAKECOL15(255,255,255)
#define NORMAL_COL		MAKECOL15(200,200,200)

#define SHADOW_LEVEL	4


void file_browser(const char *arg)
{
	int i, sel = 0, rows = 12, top = 0, up = 0;
	int title_loaded = -1, run_neogeo = 0;
	int light_level = 0, light_dir = 1, prev_sel = 0;
	char path[MAX_PATH], *p;

	bLoop = LOOP_BROWSER;

	checkDir("config");
	checkDir("snap");
#ifdef SAVE_STATE
	checkDir("state");
#endif

	memset(launchDir, 0, sizeof(launchDir));
	strncpy(launchDir, arg, sizeof(launchDir) - 1);
	if ((p = strrchr(launchDir, '/')) != NULL) *(++p) = '\0';

	memset(files, 0, sizeof(files));
	strcpy(curr_dir, launchDir);
	getDir(curr_dir);

	load_settings();

	video_clear_screen();
	load_background();
	show_background();

	boxfill_alpha(0, 0, 479, 271, 0, 16);

	draw_dialog(240-164, 136-80, 240+164, 136+80);
	font_print_shadow_center(60, TITLE_COL, APPNAME_STR " " VERSION_STR "   by NJ", SHADOW_LEVEL);
	font_print_center(60 + 16 * 1 +  5, URL_COL,  "http://neocdz.hp.infoseek.co.jp");
	font_print_center(60 + 16 * 2 + 15, COLOR_WHITE, "PSP hardware GFX render based on");
	font_print_center(60 + 16 * 3 + 15, COLOR_WHITE, "NEOCDPSP 0.5 by yoyoFR,");
	font_print_center(60 + 16 * 4 + 15, URL_COL,  "http://yoyofr.fr.st");
	font_print_center(60 + 16 * 5 + 25, COLOR_WHITE, "Powered by unofficial PSPSDK");
	font_print_center(60 + 16 * 6 + 25, URL_COL,  "http://www.ps2dev.org");

	video_flip_screen(1);

	pad_wait_press(3000);
	pad_wait_clear();

	while (bLoop)
	{
		if (run_neogeo)
		{
			run_neogeo = 0;
			bLoop = LOOP_EXEC;

			video_clear_screen();
			set_cpu_clock();

			neogeo_main();

			scePowerSetClockFrequency(222, 222, 111);
			wait(500);

			if (!bLoop) break;

			load_background();
			getDir(curr_dir);
		}

		show_background();

		small_icon_shadow(8 + 1, 24 + 1, PATH_COL, ICON_S_MEMSTICK, SHADOW_LEVEL);
		small_icon(8, 24, PATH_COL, ICON_S_MEMSTICK);
		font_print_shadow(32, 24, PATH_COL, curr_dir, SHADOW_LEVEL);
		boxfill(0, 254, 479, 271, MAKECOL15(76,76,76));
		font_print_shadow(16, 252, INFOBAR_COL, "\x01\x02\x03\x04 Browse list   \x05 Select   \x07 Boot BIOS   \x08 Exit emulator", SHADOW_LEVEL);

		draw_scrollbar(470, 47, 479, 251, 16, nfiles, sel);

		if (prev_sel == sel)
		{
			light_level += light_dir;
			if (light_level > 31)
			{
				light_level = 31;
				light_dir = -1;
			}
			else if (light_level < 0)
			{
				light_level = 0;
				light_dir = 1;
			}

			for (i = 0; i < rows; i++)
			{
				if (top + i >= nfiles) break;

				if (files[top + i].icon == ICON_S_ZIPFILE)
				{
					if (top + i == sel)
					{
						boxfill_gradation(4, 53 + i * 16, 466, 53 + i * 16 + 15, MAKECOL15(0,0,128), 0, 15);

						small_icon_lightshadow(8, 50 + i * 16, SELECT_COL, files[top + i].icon, light_level);
						small_icon_shadow(8 + 1, 50 + i * 16 + 1, SELECT_COL, files[top + i].icon, SHADOW_LEVEL);
						small_icon(8, 50 + i * 16, SELECT_COL, files[top + i].icon);

						font_print_shadow(36, 50 + i * 16, SELECT_COL, files[top + i].name, SHADOW_LEVEL);
					}
					else
					{
						small_icon( 8, 50 + i * 16, NORMAL_COL, files[top + i].icon);
						font_print(36, 50 + i * 16, NORMAL_COL, files[top + i].name);
					}
				}
				else
				{
					if (top + i == sel)
					{
						boxfill_gradation(4, 53 + i * 16, 466, 53 + i * 16 + 15, MAKECOL15(0,0,128), 0, 15);

						small_icon_lightshadow(8, 50 + i * 16, SELECT_COL, files[top + i].icon, light_level);
						small_icon_shadow(8 + 1, 50 + i * 16 + 1, SELECT_COL, files[top + i].icon, SHADOW_LEVEL);
						small_icon(8, 50 + i * 16, SELECT_COL, files[top + i].icon);

						font_print_shadow(36, 50 + i * 16, SELECT_COL, files[top + i].name, SHADOW_LEVEL);

						if (files[top + i].has_title && title_loaded == -1)
						{
							char title_path[MAX_PATH];

							sprintf(title_path, "%s%s", curr_dir, files[top + i].name);

							if (load_title(title_path))
								title_loaded = top + i;
							else
								files[top + i].has_title = 0;
						}
					}
					else
					{
						small_icon( 8, 50 + i * 16, NORMAL_COL, files[top + i].icon);
						font_print(36, 50 + i * 16, NORMAL_COL, files[top + i].name);
					}
				}
			}

			if (pad_pressed_memory(PSP_CTRL_CIRCLE))
			{
				if (files[sel].type == FIO_SO_IFDIR)
				{
					if (!strcmp(files[sel].name, ".."))
					{
	    		    	// to upper directory
						up = 1;
					}
           		    else
           		    {
	    	   	 		// to lower directory
						strcat(curr_dir, files[sel].name);
						getDir(curr_dir);
						sel = 0;

						if (neocddir)
						{
							neocddir = 0;

							boxfill_alpha(0, 0, 479, 271, 0, 16);

							draw_dialog(240-110, 136-32, 240+110, 136+32);
							font_print_shadow_center(136-26+1, INFO_COL, "Found NEOCD Directory", SHADOW_LEVEL);
							font_print_shadow_center(136+4+1, COLOR_WHITE, "\x05 to launch, \x06 to browse", SHADOW_LEVEL);

							video_flip_screen(1);

							pad_wait_clear();

							do
							{
								pad_update();

								if (pad_pressed_memory(PSP_CTRL_CIRCLE))
								{
									strcpy(game_dir, curr_dir);
									strcpy(mp3_dir, curr_dir);
									strcat(mp3_dir, "neocd.mp3/");
									*strrchr(curr_dir, '/') = '\0';
									*(strrchr(curr_dir, '/') + 1) = '\0';

									run_neogeo = 1;
									neogeo_boot_bios = 0;
									break;
								}

						   	    if (get_app_status()) break;

							} while (!pad_pressed_memory(PSP_CTRL_CROSS));

							pad_wait_clear();
							continue;
						}
					}
				}
				else
				{
					boxfill_alpha(0, 0, 479, 271, 0, 16);

					draw_dialog(240-160, 136-44, 240+160, 136+44);
					font_print_shadow_center(136-34, WARNING_COL, "Warning : zipped game can be very slow", SHADOW_LEVEL);
					font_print_shadow_center(136-10, INFO_COL, "Try to uncompress the zip file", SHADOW_LEVEL);
					font_print_shadow_center(136+14, COLOR_WHITE, "\x05 to launch, \x06 to browse", SHADOW_LEVEL);

					video_flip_screen(1);

					pad_wait_clear();

					do
					{
						pad_update();

						if (pad_pressed_memory(PSP_CTRL_CIRCLE))
						{
							strcpy(game_dir, curr_dir);
							strcpy(mp3_dir, curr_dir);
							strcat(mp3_dir, "neocd.mp3/");
							strcat(game_dir, files[sel].name);

							run_neogeo = 1;
							neogeo_boot_bios = 0;
							break;
						}

				   	    if (get_app_status()) break;

					} while (!pad_pressed_memory(PSP_CTRL_CROSS));

					pad_wait_clear();
					continue;
				}
			}
			else if (pad_pressed_memory(PSP_CTRL_SQUARE))
   		    {
				boxfill_alpha(0, 0, 479, 271, 0, 16);

				draw_dialog(240-110, 136-32, 240+110, 136+32);
				font_print_shadow_center(136-26+1, INFO_COL, "Press \x05 to boot BIOS", SHADOW_LEVEL);
				font_print_shadow_center(136+4+1, COLOR_WHITE, "\x05 to confirm, \x06 to cancel", SHADOW_LEVEL);

				video_flip_screen(1);

				pad_wait_clear();

				do
				{
					pad_update();

					if (pad_pressed_memory(PSP_CTRL_CIRCLE))
					{
						strcpy(game_dir, curr_dir);
						strcpy(mp3_dir, curr_dir);
						strcat(mp3_dir, "neocd.mp3/");

						run_neogeo = 1;
						neogeo_boot_bios = 1;
						break;
					}

			   	    if (get_app_status()) break;

				} while (!pad_pressed_memory(PSP_CTRL_CROSS));

				pad_wait_clear();
				continue;
			}
			else if (pad_pressed_memory(PSP_CTRL_TRIANGLE))
   		    {
				boxfill_alpha(0, 0, 479, 271, 0, 16);

				draw_dialog(240-110, 136-32, 240+110, 136+32);
				font_print_shadow_center(136-26+1, INFO_COL, "Press \x05 to exit emulator", SHADOW_LEVEL);
				font_print_shadow_center(136+4+1, COLOR_WHITE, "\x05 to confirm, \x06 to cancel", SHADOW_LEVEL);

				video_flip_screen(1);

				pad_wait_clear();

				do
				{
					pad_update();

					if (pad_pressed_memory(PSP_CTRL_CIRCLE))
					{
						bLoop = 0;
						break;
					}

			   	    if (get_app_status()) break;

				} while (!pad_pressed_memory(PSP_CTRL_CROSS));

				pad_wait_clear();
				continue;
			}

			if (title_loaded == sel)
			{
				if (sel < top + rows / 2)
					show_title(317, 171);
				else
					show_title(317, 48);
			}

			draw_battery_status(1);
			ui_show_popup();
			video_flip_screen(1);
		}
		else
		{
			prev_sel = sel;
			title_loaded = -1;
			light_level = 0;
			light_dir = 1;
		}

		if (pad_pressed_memory(PSP_CTRL_UP))
        {
        	sel--;
        }
		else if (pad_pressed_memory(PSP_CTRL_DOWN))
		{
        	sel++;
		}
		else if (pad_pressed_memory(PSP_CTRL_LEFT))
		{
			sel -= 10;
			if (sel < 0) sel = 0;
		}
		else if (pad_pressed_memory(PSP_CTRL_RIGHT))
		{
        	sel += 10;
        	if (sel >= nfiles) sel = nfiles - 1;
		}

		if (top > nfiles - rows) top = nfiles - rows;
		if (top < 0) top = 0;
		if (sel >= nfiles) sel = 0;
		if (sel < 0) sel = nfiles - 1;
		if (sel >= top + rows) top = sel - rows + 1;
		if (sel < top) top = sel;

		if (up)
		{
			if (strcmp(curr_dir, "ms0:/") != 0)
			{
				char old_dir[MAX_PATH];

				*strrchr(curr_dir, '/') = '\0';
				p = strrchr(curr_dir, '/') + 1;
				strcpy(old_dir, p);
				strcat(old_dir,  "/");
				*p = '\0';

				getDir(curr_dir);

				sel = 0;
				for (i = 0; i < nfiles; i++)
				{
					if (!strcmp(old_dir, files[i].name))
					{
						sel = i;
						top = sel - 3;
						break;
					}
				}
			}
			up = 0;
			pad_wait_clear();
		}

		pad_update();
	}

	save_settings();
}
